@@ -4,19 +4,17 @@
 
     Test the HTML builder and check output against XPath.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import os
 import subprocess
-from subprocess import CalledProcessError, PIPE
+from subprocess import PIPE, CalledProcessError
 from xml.etree import ElementTree
 
 import pytest
 
-from sphinx.util import docutils
-
 
 # check given command is runnable
 def runnable(command):
@@ -67,11 +65,11 @@ class EPUBElementTree:
 @pytest.mark.sphinx('epub', testroot='basic')
 def test_build_epub(app):
     app.build()
-    assert (app.outdir / 'mimetype').text() == 'application/epub+zip'
+    assert (app.outdir / 'mimetype').read_text() == 'application/epub+zip'
     assert (app.outdir / 'META-INF' / 'container.xml').exists()
 
     # toc.ncx
-    toc = EPUBElementTree.fromstring((app.outdir / 'toc.ncx').text())
+    toc = EPUBElementTree.fromstring((app.outdir / 'toc.ncx').read_text())
     assert toc.find("./ncx:docTitle/ncx:text").text == 'Python'
 
     # toc.ncx / head
@@ -91,7 +89,7 @@ def test_build_epub(app):
     assert navlabel.text == 'The basic Sphinx documentation for testing'
 
     # content.opf
-    opf = EPUBElementTree.fromstring((app.outdir / 'content.opf').text())
+    opf = EPUBElementTree.fromstring((app.outdir / 'content.opf').read_text())
 
     # content.opf / metadata
     metadata = opf.find("./idpf:metadata")
@@ -143,7 +141,7 @@ def test_build_epub(app):
     assert reference.get('href') == 'index.xhtml'
 
     # nav.xhtml
-    nav = EPUBElementTree.fromstring((app.outdir / 'nav.xhtml').text())
+    nav = EPUBElementTree.fromstring((app.outdir / 'nav.xhtml').read_text())
     assert nav.attrib == {'lang': 'en',
                           '{http://www.w3.org/XML/1998/namespace}lang': 'en'}
     assert nav.find("./xhtml:head/xhtml:title").text == 'Table of Contents'
@@ -163,7 +161,7 @@ def test_epub_cover(app):
     app.build()
 
     # content.opf / metadata
-    opf = EPUBElementTree.fromstring((app.outdir / 'content.opf').text())
+    opf = EPUBElementTree.fromstring((app.outdir / 'content.opf').read_text())
     cover_image = opf.find("./idpf:manifest/idpf:item[@href='%s']" % app.config.epub_cover[0])
     cover = opf.find("./idpf:metadata/idpf:meta[@name='cover']")
     assert cover
@@ -175,7 +173,7 @@ def test_nested_toc(app):
     app.build()
 
     # toc.ncx
-    toc = EPUBElementTree.fromstring((app.outdir / 'toc.ncx').bytes())
+    toc = EPUBElementTree.fromstring((app.outdir / 'toc.ncx').read_bytes())
     assert toc.find("./ncx:docTitle/ncx:text").text == 'Python'
 
     # toc.ncx / navPoint
@@ -205,7 +203,7 @@ def test_nested_toc(app):
         anchor = elem.find("./xhtml:a")
         return (anchor.get('href'), anchor.text)
 
-    nav = EPUBElementTree.fromstring((app.outdir / 'nav.xhtml').bytes())
+    nav = EPUBElementTree.fromstring((app.outdir / 'nav.xhtml').read_bytes())
     toc = nav.findall("./xhtml:body/xhtml:nav/xhtml:ol/xhtml:li")
     assert len(toc) == 4
     assert navinfo(toc[0]) == ('index.xhtml',
@@ -230,7 +228,7 @@ def test_escaped_toc(app):
     app.build()
 
     # toc.ncx
-    toc = EPUBElementTree.fromstring((app.outdir / 'toc.ncx').bytes())
+    toc = EPUBElementTree.fromstring((app.outdir / 'toc.ncx').read_bytes())
     assert toc.find("./ncx:docTitle/ncx:text").text == 'need <b>"escaped"</b> project'
 
     # toc.ncx / navPoint
@@ -260,7 +258,7 @@ def test_escaped_toc(app):
         anchor = elem.find("./xhtml:a")
         return (anchor.get('href'), anchor.text)
 
-    nav = EPUBElementTree.fromstring((app.outdir / 'nav.xhtml').bytes())
+    nav = EPUBElementTree.fromstring((app.outdir / 'nav.xhtml').read_bytes())
     toc = nav.findall("./xhtml:body/xhtml:nav/xhtml:ol/xhtml:li")
     assert len(toc) == 4
     assert navinfo(toc[0]) == ('index.xhtml',
@@ -286,7 +284,7 @@ def test_epub_writing_mode(app):
     app.build()
 
     # horizontal / page-progression-direction
-    opf = EPUBElementTree.fromstring((app.outdir / 'content.opf').text())
+    opf = EPUBElementTree.fromstring((app.outdir / 'content.opf').read_text())
     assert opf.find("./idpf:spine").get('page-progression-direction') == 'ltr'
 
     # horizontal / ibooks:scroll-axis
@@ -294,7 +292,7 @@ def test_epub_writing_mode(app):
     assert metadata.find("./idpf:meta[@property='ibooks:scroll-axis']").text == 'vertical'
 
     # horizontal / writing-mode (CSS)
-    css = (app.outdir / '_static' / 'epub.css').text()
+    css = (app.outdir / '_static' / 'epub.css').read_text()
     assert 'writing-mode: horizontal-tb;' in css
 
     # vertical
@@ -303,7 +301,7 @@ def test_epub_writing_mode(app):
     app.build()
 
     # vertical / page-progression-direction
-    opf = EPUBElementTree.fromstring((app.outdir / 'content.opf').text())
+    opf = EPUBElementTree.fromstring((app.outdir / 'content.opf').read_text())
     assert opf.find("./idpf:spine").get('page-progression-direction') == 'rtl'
 
     # vertical / ibooks:scroll-axis
@@ -311,7 +309,7 @@ def test_epub_writing_mode(app):
     assert metadata.find("./idpf:meta[@property='ibooks:scroll-axis']").text == 'horizontal'
 
     # vertical / writing-mode (CSS)
-    css = (app.outdir / '_static' / 'epub.css').text()
+    css = (app.outdir / '_static' / 'epub.css').read_text()
     assert 'writing-mode: vertical-rl;' in css
 
 
@@ -319,9 +317,11 @@ def test_epub_writing_mode(app):
 def test_epub_anchor_id(app):
     app.build()
 
-    html = (app.outdir / 'index.xhtml').text()
-    assert '<p id="std-setting-STATICFILES_FINDERS">blah blah blah</p>' in html
-    assert '<span id="std-setting-STATICFILES_SECTION"></span><h1>blah blah blah</h1>' in html
+    html = (app.outdir / 'index.xhtml').read_text()
+    assert ('<p id="std-setting-STATICFILES_FINDERS">'
+            'blah blah blah</p>' in html)
+    assert ('<span id="std-setting-STATICFILES_SECTION"></span>'
+            '<h1>blah blah blah</h1>' in html)
     assert 'see <a class="reference internal" href="#std-setting-STATICFILES_FINDERS">' in html
 
 
@@ -330,7 +330,7 @@ def test_epub_assets(app):
     app.builder.build_all()
 
     # epub_sytlesheets (same as html_css_files)
-    content = (app.outdir / 'index.xhtml').text()
+    content = (app.outdir / 'index.xhtml').read_text()
     assert ('<link rel="stylesheet" type="text/css" href="_static/css/style.css" />'
             in content)
     assert ('<link media="print" rel="stylesheet" title="title" type="text/css" '
@@ -343,7 +343,7 @@ def test_epub_css_files(app):
     app.builder.build_all()
 
     # epub_css_files
-    content = (app.outdir / 'index.xhtml').text()
+    content = (app.outdir / 'index.xhtml').read_text()
     assert '<link rel="stylesheet" type="text/css" href="_static/css/epub.css" />' in content
 
     # files in html_css_files are not outputed
@@ -353,14 +353,12 @@ def test_epub_css_files(app):
             'href="https://example.com/custom.css" />' not in content)
 
 
-@pytest.mark.skipif(docutils.__version_info__ < (0, 13),
-                    reason='docutils-0.13 or above is required')
 @pytest.mark.sphinx('epub', testroot='roles-download')
 def test_html_download_role(app, status, warning):
     app.build()
     assert not (app.outdir / '_downloads' / 'dummy.dat').exists()
 
-    content = (app.outdir / 'index.xhtml').text()
+    content = (app.outdir / 'index.xhtml').read_text()
     assert ('<li><p><code class="xref download docutils literal notranslate">'
             '<span class="pre">dummy.dat</span></code></p></li>' in content)
     assert ('<li><p><code class="xref download docutils literal notranslate">'
@@ -389,6 +387,6 @@ def test_run_epubcheck(app):
             subprocess.run(['java', '-jar', epubcheck, app.outdir / 'SphinxTests.epub'],
                            stdout=PIPE, stderr=PIPE, check=True)
         except CalledProcessError as exc:
-            print(exc.stdout)
-            print(exc.stderr)
+            print(exc.stdout.decode('utf-8'))
+            print(exc.stderr.decode('utf-8'))
             assert False, 'epubcheck exited with return code %s' % exc.returncode
