@@ -74,7 +74,7 @@ def test_mangle_signature():
 
 
 def test_extract_summary(capsys):
-    settings = Mock(language_code='',
+    settings = Mock(language_code='en',
                     id_prefix='',
                     auto_id_prefix='',
                     pep_reference=False,
@@ -161,6 +161,7 @@ def test_get_items_summary(make_app, app_params):
         'emptyLine': "This is the real summary",
         'module_attr': 'This is a module attribute',
         'C.class_attr': 'This is a class attribute',
+        'C.instance_attr': 'This is an instance attribute',
         'C.prop_attr1': 'This is a function docstring',
         'C.prop_attr2': 'This is a attribute docstring',
         'C.C2': 'This is a nested inner class docstring',
@@ -329,6 +330,7 @@ def test_autosummary_generate(app, status, warning):
             '      ~Foo.CONSTANT3\n'
             '      ~Foo.CONSTANT4\n'
             '      ~Foo.baz\n'
+            '      ~Foo.value\n'
             '   \n' in Foo)
 
     FooBar = (app.srcdir / 'generated' / 'autosummary_dummy_module.Foo.Bar.rst').read_text()
