@@ -4,15 +4,12 @@
 
     Test pycode.parser.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-import sys
-
-import pytest
-
 from sphinx.pycode.parser import Parser
+from sphinx.util.inspect import signature_from_str
 
 
 def test_comment_picker_basic():
@@ -94,17 +91,23 @@ def test_comment_picker_location():
                                ('Foo', 'attr3'): 'comment for attr3(3)'}
 
 
-@pytest.mark.skipif(sys.version_info < (3, 6), reason='tests for py36+ syntax')
-def test_annotated_assignment_py36():
+def test_annotated_assignment():
     source = ('a: str = "Sphinx"  #: comment\n'
               'b: int = 1\n'
               '"""string on next line"""\n'
-              'c: int  #: comment')
+              'c: int  #: comment\n'
+              'd = 1  # type: int\n'
+              '"""string on next line"""\n')
     parser = Parser(source)
     parser.parse()
     assert parser.comments == {('', 'a'): 'comment',
                                ('', 'b'): 'string on next line',
-                               ('', 'c'): 'comment'}
+                               ('', 'c'): 'comment',
+                               ('', 'd'): 'string on next line'}
+    assert parser.annotations == {('', 'a'): 'str',
+                                  ('', 'b'): 'int',
+                                  ('', 'c'): 'int',
+                                  ('', 'd'): 'int'}
     assert parser.definitions == {}
 
 
@@ -367,3 +370,158 @@ def test_formfeed_char():
     parser = Parser(source)
     parser.parse()
     assert parser.comments == {('Foo', 'attr'): 'comment'}
+
+
+def test_typing_final():
+    source = ('import typing\n'
+              '\n'
+              '@typing.final\n'
+              'def func(): pass\n'
+              '\n'
+              '@typing.final\n'
+              'class Foo:\n'
+              '    @typing.final\n'
+              '    def meth(self):\n'
+              '        pass\n')
+    parser = Parser(source)
+    parser.parse()
+    assert parser.finals == ['func', 'Foo', 'Foo.meth']
+
+
+def test_typing_final_from_import():
+    source = ('from typing import final\n'
+              '\n'
+              '@final\n'
+              'def func(): pass\n'
+              '\n'
+              '@final\n'
+              'class Foo:\n'
+              '    @final\n'
+              '    def meth(self):\n'
+              '        pass\n')
+    parser = Parser(source)
+    parser.parse()
+    assert parser.finals == ['func', 'Foo', 'Foo.meth']
+
+
+def test_typing_final_import_as():
+    source = ('import typing as foo\n'
+              '\n'
+              '@foo.final\n'
+              'def func(): pass\n'
+              '\n'
+              '@foo.final\n'
+              'class Foo:\n'
+              '    @typing.final\n'
+              '    def meth(self):\n'
+              '        pass\n')
+    parser = Parser(source)
+    parser.parse()
+    assert parser.finals == ['func', 'Foo']
+
+
+def test_typing_final_from_import_as():
+    source = ('from typing import final as bar\n'
+              '\n'
+              '@bar\n'
+              'def func(): pass\n'
+              '\n'
+              '@bar\n'
+              'class Foo:\n'
+              '    @final\n'
+              '    def meth(self):\n'
+              '        pass\n')
+    parser = Parser(source)
+    parser.parse()
+    assert parser.finals == ['func', 'Foo']
+
+
+def test_typing_final_not_imported():
+    source = ('@typing.final\n'
+              'def func(): pass\n'
+              '\n'
+              '@typing.final\n'
+              'class Foo:\n'
+              '    @final\n'
+              '    def meth(self):\n'
+              '        pass\n')
+    parser = Parser(source)
+    parser.parse()
+    assert parser.finals == []
+
+
+def test_typing_overload():
+    source = ('import typing\n'
+              '\n'
+              '@typing.overload\n'
+              'def func(x: int, y: int) -> int: pass\n'
+              '\n'
+              '@typing.overload\n'
+              'def func(x: str, y: str) -> str: pass\n'
+              '\n'
+              'def func(x, y): pass\n')
+    parser = Parser(source)
+    parser.parse()
+    assert parser.overloads == {'func': [signature_from_str('(x: int, y: int) -> int'),
+                                         signature_from_str('(x: str, y: str) -> str')]}
+
+
+def test_typing_overload_from_import():
+    source = ('from typing import overload\n'
+              '\n'
+              '@overload\n'
+              'def func(x: int, y: int) -> int: pass\n'
+              '\n'
+              '@overload\n'
+              'def func(x: str, y: str) -> str: pass\n'
+              '\n'
+              'def func(x, y): pass\n')
+    parser = Parser(source)
+    parser.parse()
+    assert parser.overloads == {'func': [signature_from_str('(x: int, y: int) -> int'),
+                                         signature_from_str('(x: str, y: str) -> str')]}
+
+
+def test_typing_overload_import_as():
+    source = ('import typing as foo\n'
+              '\n'
+              '@foo.overload\n'
+              'def func(x: int, y: int) -> int: pass\n'
+              '\n'
+              '@foo.overload\n'
+              'def func(x: str, y: str) -> str: pass\n'
+              '\n'
+              'def func(x, y): pass\n')
+    parser = Parser(source)
+    parser.parse()
+    assert parser.overloads == {'func': [signature_from_str('(x: int, y: int) -> int'),
+                                         signature_from_str('(x: str, y: str) -> str')]}
+
+
+def test_typing_overload_from_import_as():
+    source = ('from typing import overload as bar\n'
+              '\n'
+              '@bar\n'
+              'def func(x: int, y: int) -> int: pass\n'
+              '\n'
+              '@bar\n'
+              'def func(x: str, y: str) -> str: pass\n'
+              '\n'
+              'def func(x, y): pass\n')
+    parser = Parser(source)
+    parser.parse()
+    assert parser.overloads == {'func': [signature_from_str('(x: int, y: int) -> int'),
+                                         signature_from_str('(x: str, y: str) -> str')]}
+
+
+def test_typing_overload_not_imported():
+    source = ('@typing.final\n'
+              'def func(x: int, y: int) -> int: pass\n'
+              '\n'
+              '@typing.final\n'
+              'def func(x: str, y: str) -> str: pass\n'
+              '\n'
+              'def func(x, y): pass\n')
+    parser = Parser(source)
+    parser.parse()
+    assert parser.overloads == {}
